/****************  Includes  ****************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/****************  Macros  ****************/


#define REPERT_CHAMP_TAILLE 128  /* Taille des champs du répertoire  */


/****************  Type definitions  ****************/


typedef struct RepertCarte_ {
  char nom[64];     /*  Nom de la personne à rechercher  */
  char numero[64];  /*  Numéro de téléphone associé     */
} RepertCarte;


typedef struct RepertListe_ {
  struct RepertListe_ * suivant;  /*  Pointeur sur élément suivant  */
  RepertCarte carte;              /*  Carte du répertoire  */
} RepertListe;


typedef RepertListe * Repert;  /*  Définition du répertoire  */


/****************  Static function prototypes  ****************/


static int repertInit(Repert * repert);
static int repertCherche(Repert * repert, char * nom, char * numero);
static int repertAjoute(Repert * repert, char * nom, char * numero);


/****************  Main function  ****************/


int
main(int argc, char * argv[])
{
  Repert repert;                            /*  Le répertoire courant  */
  char nom[64];                             /*  Le nom courant  */
  char numero[64];                          /*  Le numéro courant  */
  char numero_trouve[REPERT_CHAMP_TAILLE];  /*  Le numéro trouvé  */


  repertInit(&repert);


  while (1) {
    /*  Récupération du nom  */
    printf("Entrez le nom : ");
    scanf("%63s", nom);
    nom[63] = '\0';
    /* Si on entre . rien ne sera fait */
    /* Si on entre + on ajoutera une nouvelle carte*/
    /* Sinon on regarde une carte*/


    /* Si phase d'ajout  */
    if (strcmp(nom, "+") == 0)   
      {
      /*  Récupération des informations  */
      printf("Entrez le nom à ajouter : ");
      scanf("%63s", nom);
      nom[63] = '\0';
      printf("Entrez le numéro associé : ");
      scanf("%63s", numero);
      numero[63] = '\0';

      /*  Ajout de la nouvelle carte  */
      if (repertAjoute(&repert, nom, numero) != 0) {
        printf("Désolé, je n'ai pas pu ajouter le nom.\n");
      }
    }
    /*  Terminaison  */
    else if (strcmp(nom, ".") == 0)
      {
	return (0);
      }
    /*  Phase de consultation  */
    else {
      /*  Si carte pas trouvée  */
      if (repertCherche(&repert, nom, numero_trouve) != 0)
	{
	  printf("Désolé, je ne connais pas le nom \"%s\".\n", nom);
	}
      /*  Si carte trouvée  */
      else
	{
	  printf("Le numéro de téléphone de \"%s\" est : %s.\n",
		 nom,
		 numero_trouve);
	}
    }
  }
}


/****************  Static functions  ****************/


static int
repertInit(Repert * repert)
{
  /* Dans cette implémentation, la liste est vide */
  *repert = NULL;
}


static int
repertCherche(Repert * repert,
	      char * nom,       /*  Nom à rechercher  */
	      char * numero)    /*  Numéro trouvé     */
{
  RepertListe * liste;  /*  Le répertoire courant  */

  for (liste = *repert; liste != NULL; liste = liste->suivant) {
    /*  Si nom recherché trouvé  */
    if (strcmp(liste->carte.nom, nom) == 0) {
      strcpy(numero, liste->carte.numero);
      /*  Nom trouvé  */
      return (0);
    }
  }

  /*  Nom pas trouvé  */
  return (1);
}


static int
repertAjoute(Repert * repert,
	     char * nom,       /*  Nom à ajouter  */
	     char * numero)    /*  Numéro à ajouter  */
{
  RepertListe * nouveau;  /*  Le répertoire courant   */
  RepertListe * liste;

  if ((nouveau = malloc(sizeof(RepertListe))) == NULL) {
    /*  Si pas possible de créer liste  */
    return (1);
  }

  strncpy(nouveau->carte.nom, nom, REPERT_CHAMP_TAILLE);
  nouveau->carte.nom[REPERT_CHAMP_TAILLE - 1] = '\0';
  strncpy(nouveau->carte.numero, numero, REPERT_CHAMP_TAILLE);
  nouveau->carte.numero[REPERT_CHAMP_TAILLE - 1] = '\0';

  liste = *repert;            /*  Récupère pointeur sur le premier élément  */
  nouveau->suivant = liste;
  *repert = nouveau;

  /*  Tout s'est bien passé  */
  return (0);
}
